﻿using AutoWorkshopBusinessLogic.OfficePackage.HelperEnums;
using AutoWorkshopBusinessLogic.OfficePackage.HelperModels;

namespace AutoWorkshopBusinessLogic.OfficePackage
{
    public abstract class AbstractSaveToExcel
    {
        public void CreateReport(ExcelRepairsInfo Info)
        {
            CreateExcel(Info);

            InsertCellInWorksheet(new ExcelCellParameters
            {
                ColumnName = "A",
                RowIndex = 1,
                Text = Info.Title,
                StyleInfo = ExcelStyleInfoType.Title
            });

            MergeCells(new ExcelMergeParameters
            {
                CellFromName = "A1",
                CellToName = "C1"
            });

            uint RowIndex = 2;
            
            foreach (var RepComp in Info.RepairComponents)
            {
                InsertCellInWorksheet(new ExcelCellParameters
                {
                    ColumnName = "A",
                    RowIndex = RowIndex,
                    Text = RepComp.RepairName,
                    StyleInfo = ExcelStyleInfoType.Text
                });
                
                RowIndex++;

                foreach (var (Component, Count) in RepComp.Components)
                {
                    InsertCellInWorksheet(new ExcelCellParameters
                    {
                        ColumnName = "B",
                        RowIndex = RowIndex,
                        Text = Component,
                        StyleInfo = ExcelStyleInfoType.TextWithBroder
                    });

                    InsertCellInWorksheet(new ExcelCellParameters
                    {
                        ColumnName = "C",
                        RowIndex = RowIndex,
                        Text = Count.ToString(),
                        StyleInfo = ExcelStyleInfoType.TextWithBroder
                    });

                    RowIndex++;
                }

                InsertCellInWorksheet(new ExcelCellParameters
                {
                    ColumnName = "A",
                    RowIndex = RowIndex,
                    Text = "Итого",
                    StyleInfo = ExcelStyleInfoType.Text
                });
                
                InsertCellInWorksheet(new ExcelCellParameters
                {
                    ColumnName = "C",
                    RowIndex = RowIndex,
                    Text = RepComp.TotalCount.ToString(),
                    StyleInfo = ExcelStyleInfoType.Text
                });
                RowIndex++;
            }

            SaveExcel(Info);
        }

        public void CreateShopRepairsReport(ExcelShopsInfo Info)
        {
            CreateExcel(Info);

            InsertCellInWorksheet(new ExcelCellParameters
            {
                ColumnName = "A",
                RowIndex = 1,
                Text = Info.Title,
                StyleInfo = ExcelStyleInfoType.Title
            });

            MergeCells(new ExcelMergeParameters
            {
                CellFromName = "A1",
                CellToName = "C1"
            });

            uint RowIndex = 2;

            foreach (var ShopRep in Info.ShopRepairs)
            {
                InsertCellInWorksheet(new ExcelCellParameters
                {
                    ColumnName = "A",
                    RowIndex = RowIndex,
                    Text = ShopRep.ShopName,
                    StyleInfo = ExcelStyleInfoType.Text
                });

                RowIndex++;

                foreach (var (Repair, Count) in ShopRep.Repairs)
                {
                    InsertCellInWorksheet(new ExcelCellParameters
                    {
                        ColumnName = "B",
                        RowIndex = RowIndex,
                        Text = Repair,
                        StyleInfo = ExcelStyleInfoType.TextWithBroder
                    });

                    InsertCellInWorksheet(new ExcelCellParameters
                    {
                        ColumnName = "C",
                        RowIndex = RowIndex,
                        Text = Count.ToString(),
                        StyleInfo = ExcelStyleInfoType.TextWithBroder
                    });

                    RowIndex++;
                }

                InsertCellInWorksheet(new ExcelCellParameters
                {
                    ColumnName = "A",
                    RowIndex = RowIndex,
                    Text = "Итого",
                    StyleInfo = ExcelStyleInfoType.Text
                });
                
                InsertCellInWorksheet(new ExcelCellParameters
                {
                    ColumnName = "C",
                    RowIndex = RowIndex,
                    Text = ShopRep.TotalCount.ToString(),
                    StyleInfo = ExcelStyleInfoType.Text
                });
                
                RowIndex++;
            }

            SaveExcel(Info);
        }

        protected abstract void CreateExcel(IDocumentInfo Info);
        
        protected abstract void InsertCellInWorksheet(ExcelCellParameters ExcelParams);
        
        protected abstract void MergeCells(ExcelMergeParameters ExcelParams);
        
        protected abstract void SaveExcel(IDocumentInfo Info);
    }
}
