﻿using AutoWorkshopBusinessLogic.OfficePackage.HelperEnums;
using AutoWorkshopBusinessLogic.OfficePackage.HelperModels;

namespace AutoWorkshopBusinessLogic.OfficePackage
{
    public abstract class AbstractSaveToWord
    {
        public void CreateRepairsDoc(WordRepairsInfo Info)
        {
            CreateWord(Info);

            CreateParagraph(new WordParagraph
            {
                Texts = new List<(string, WordTextProperties)> { (Info.Title, new WordTextProperties { Bold = true, Size = "24", }) },
                TextProperties = new WordTextProperties
                {
                    Size = "24",
                    JustificationType = WordJustificationType.Center
                }
            });

            foreach (var Repair in Info.Repairs)
            {
                CreateParagraph(new WordParagraph
                {
                    Texts = new List<(string, WordTextProperties)> {
                        (Repair.RepairName, new WordTextProperties { Size = "24", Bold = true}),
                        ("\t"+Repair.Price.ToString(), new WordTextProperties{Size = "24"})
                    },
                    TextProperties = new WordTextProperties
                    {
                        Size = "24",
                        JustificationType = WordJustificationType.Both
                    }
                });
            }

            SaveWord(Info);
        }

        public void CreateShopsDoc(WordShopsInfo Info)
        {
            CreateWord(Info);
            
            CreateParagraph(new WordParagraph
            {
                Texts = new List<(string, WordTextProperties)> { (Info.Title, new WordTextProperties { Bold = true, Size = "24", }) },
                TextProperties = new WordTextProperties
                {
                    Size = "24",
                    JustificationType = WordJustificationType.Center
                }
            });

            CreateTable(new List<string> { "3000", "3000", "3000" });
            CreateRow(new WordRowParameters
            {
                Texts = new List<string> { "Название", "Адрес", "Дата открытия" },
                TextProperties = new WordTextProperties
                {
                    Size = "24",
                    Bold = true,
                    JustificationType = WordJustificationType.Center
                }
            });

            foreach (var Shop in Info.Shops)
            {
                CreateRow(new WordRowParameters
                {
                    Texts = new List<string> { Shop.ShopName, Shop.Address, Shop.OpeningDate.ToString() },
                    TextProperties = new WordTextProperties
                    {
                        Size = "22",
                        JustificationType = WordJustificationType.Both
                    }
                });
            }

            SaveWord(Info);
        }

        protected abstract void CreateWord(IDocumentInfo Info);
        
        protected abstract void CreateParagraph(WordParagraph Paragraph);
        
        protected abstract void SaveWord(IDocumentInfo Info);

        protected abstract void CreateTable(List<string> Colums);
        
        protected abstract void CreateRow(WordRowParameters RowParameters);
    }
}
