﻿using AutoWorkshopContracts.BindingModels;
using AutoWorkshopContracts.SearchModels;
using AutoWorkshopContracts.StoragesContracts;
using AutoWorkshopContracts.ViewModels;
using AutoWorkshopDatabaseImplement.Models;
using Microsoft.EntityFrameworkCore;

namespace AutoWorkshopDatabaseImplement.Implements
{
    public class RepairStorage : IRepairStorage
    {
        public List<RepairViewModel> GetFullList()
        {
            using var Context = new AutoWorkshopDatabase();
            return Context.Repairs
                .Include(x => x.Components)
                .ThenInclude(x => x.Component)
                .ToList()
                .Select(x => x.GetViewModel)
                .ToList();
        }
        
        public List<RepairViewModel> GetFilteredList(RepairSearchModel Model)
        {
            if (string.IsNullOrEmpty(Model.RepairName))
                return new();

            using var Context = new AutoWorkshopDatabase();
            
            return Context.Repairs
                .Include(x => x.Components)
                .ThenInclude(x => x.Component)
                .Where(x => x.RepairName.Contains(Model.RepairName))
                .ToList()
                .Select(x => x.GetViewModel)
                .ToList();
        }
       
        public RepairViewModel? GetElement(RepairSearchModel Model)
        {
            if (string.IsNullOrEmpty(Model.RepairName) && !Model.Id.HasValue)
                return null;
            
            using var Context = new AutoWorkshopDatabase();
            
            return Context.Repairs
                .Include(x => x.Components)
                .ThenInclude(x => x.Component)
                .FirstOrDefault(x => (!string.IsNullOrEmpty(Model.RepairName) && x.RepairName == Model.RepairName) ||
                (Model.Id.HasValue && x.Id == Model.Id))?
                .GetViewModel;
        }
        
        public RepairViewModel? Insert(RepairBindingModel Model)
        {
            using var Context = new AutoWorkshopDatabase();
            
            var NewRepair = Repair.Create(Context, Model);
            if (NewRepair == null)
                return null;

            Context.Repairs.Add(NewRepair);
            Context.SaveChanges();
            
            return NewRepair.GetViewModel;
        }
        
        public RepairViewModel? Update(RepairBindingModel Model)
        {
            using var Context = new AutoWorkshopDatabase();

            using var Transaction = Context.Database.BeginTransaction();            
            try
            {
                var Repair = Context.Repairs.FirstOrDefault(rec => rec.Id == Model.Id);
                
                if (Repair == null)
                    return null;

                Repair.Update(Model);
                Context.SaveChanges();
                Repair.UpdateComponents(Context, Model);
                
                Transaction.Commit();
                return Repair.GetViewModel;
            }
            catch
            {
                Transaction.Rollback();
                throw;
            }
        }
        public RepairViewModel? Delete(RepairBindingModel Model)
        {
            using var Context = new AutoWorkshopDatabase();
            
            var Repair = Context.Repairs
                .Include(x => x.Components)
                .FirstOrDefault(rec => rec.Id == Model.Id);

            if (Repair == null)
                return null;

            Context.Repairs.Remove(Repair);
            Context.SaveChanges();
            return Repair.GetViewModel;
        }
    }
}
