﻿using AutoWorkshopContracts.BindingModels;
using AutoWorkshopContracts.ViewModels;
using AutoWorkshopDataModels.Enums;
using AutoWorkshopDataModels.Models;
using System.ComponentModel.DataAnnotations;

namespace AutoWorkshopDatabaseImplement.Models
{
    public class Order : IOrderModel
    {
        public int Id { get; private set; }
        
        [Required]
        public int RepairId { get; private set; }

        public virtual Repair Repair { get; set; }

        [Required]
        public int ClientId { get; private set; }
        
        public virtual Client Client { get; set; }

        [Required]
        public int Count { get; private set; }
        
        [Required]
        public double Sum { get; private set; }
        
        [Required]
        public OrderStatus Status { get; private set; } = OrderStatus.Undefined;
        
        [Required]
        public DateTime DateCreate { get; private set; } = DateTime.Now;
        
        public DateTime? DateImplement { get; private set; }
        
        public static Order? Create(OrderBindingModel Model)
        {
            if (Model is null)
                return null;

            return new Order()
            {
                Id = Model.Id,
                RepairId = Model.RepairId,
                ClientId = Model.ClientId,
                Count = Model.Count,
                Sum = Model.Sum,
                Status = Model.Status,
                DateCreate = Model.DateCreate,
                DateImplement = Model.DateImplement,
            };
        }
        
        public void Update(OrderBindingModel Model)
        {
            if (Model is null)
                return;

            Status = Model.Status;
            DateImplement = Model.DateImplement;
        }
        
        public OrderViewModel GetViewModel => new()
        {
            Id = Id,
            RepairId = RepairId,
            ClientId = ClientId,
            RepairName = Repair.RepairName,
            ClientFIO = Client.ClientFIO,
            Count = Count,
            Sum = Sum,
            Status = Status,
            DateCreate = DateCreate,
            DateImplement = DateImplement,
        };
    }
}
