﻿using AutoWorkshopContracts.BindingModels;
using AutoWorkshopContracts.ViewModels;
using AutoWorkshopDataModels.Models;
using System.ComponentModel.DataAnnotations;
using System.ComponentModel.DataAnnotations.Schema;

namespace AutoWorkshopDatabaseImplement.Models
{
    public class Repair : IRepairModel
    {
        public int Id { get; set; }
        
        [Required]
        public string RepairName { get; set; } = string.Empty;
        
        [Required]
        public double Price { get; set; }
        
        private Dictionary<int, (IComponentModel, int)>? _repairComponents = null;
        
        [NotMapped]
        public Dictionary<int, (IComponentModel, int)> RepairComponents
        {
            get
            {
                if (_repairComponents == null)
                {
                    _repairComponents = Components.ToDictionary(RepComp => RepComp.ComponentId, RepComp => 
                        (RepComp.Component as IComponentModel, RepComp.Count));
                }
                
                return _repairComponents;
            }
        }
        
        [ForeignKey("RepairId")]
        public virtual List<RepairComponent> Components { get; set; } = new();
        
        [ForeignKey("RepairId")]
        public virtual List<Order> Orders { get; set; } = new();

        [ForeignKey("RepairId")]
        public virtual List<ShopRepair> Shops { get; set; } = new();

        public static Repair Create(AutoWorkshopDatabase Context, RepairBindingModel Model)
        {
            return new Repair()
            {
                Id = Model.Id,
                RepairName = Model.RepairName,
                Price = Model.Price,
                Components = Model.RepairComponents.Select(x => new RepairComponent
                {
                    Component = Context.Components.First(y => y.Id == x.Key),
                    Count = x.Value.Item2
                }).ToList()
            };
        }
        
        public void Update(RepairBindingModel Model)
        {
            RepairName = Model.RepairName;
            Price = Model.Price;
        }
        
        public RepairViewModel GetViewModel => new()
        {
            Id = Id,
            RepairName = RepairName,
            Price = Price,
            RepairComponents = RepairComponents
        };
        
        public void UpdateComponents(AutoWorkshopDatabase Context, RepairBindingModel Model)
        {
            var RepairComponents = Context.RepairComponents.Where(rec => rec.RepairId == Model.Id).ToList();
            
            if (RepairComponents != null && RepairComponents.Count > 0)
            {
                Context.RepairComponents.RemoveRange(RepairComponents.Where(rec => !Model.RepairComponents.ContainsKey(rec.ComponentId)));
                Context.SaveChanges();

                foreach (var ComponentToUpdate in RepairComponents)
                {
                    ComponentToUpdate.Count = Model.RepairComponents[ComponentToUpdate.ComponentId].Item2;
                    Model.RepairComponents.Remove(ComponentToUpdate.ComponentId);
                }

                Context.SaveChanges();
            }

            var Repair = Context.Repairs.First(x => x.Id == Id);
            
            foreach (var RepairComponent in Model.RepairComponents)
            {
                Context.RepairComponents.Add(new RepairComponent
                {
                    Repair = Repair,
                    Component = Context.Components.First(x => x.Id == RepairComponent.Key),
                    Count = RepairComponent.Value.Item2
                });
                
                Context.SaveChanges();
            }
            
            _repairComponents = null;
        }
    }
}
