﻿using AutoWorkshopContracts.BindingModels;
using AutoWorkshopContracts.SearchModels;
using AutoWorkshopContracts.StoragesContracts;
using AutoWorkshopContracts.ViewModels;
using AutoWorkshopFileImplement.Models;

namespace AutoWorkshopFileImplement.Implements
{
    public class RepairStorage : IRepairStorage
    {
        private readonly DataFileSingleton _source;
        
        public RepairStorage()
        {
            _source = DataFileSingleton.GetInstance();
        }
        
        public List<RepairViewModel> GetFullList()
        {
            return _source.Repairs.Select(x => x.GetViewModel).ToList();
        }
        
        public List<RepairViewModel> GetFilteredList(RepairSearchModel Model)
        {
            if (string.IsNullOrEmpty(Model.RepairName))
                return new();

            return _source.Repairs.Where(x => x.RepairName.Contains(Model.RepairName)).Select(x => x.GetViewModel).ToList();
        }
        
        public RepairViewModel? GetElement(RepairSearchModel Model)
        {
            if (string.IsNullOrEmpty(Model.RepairName) && !Model.Id.HasValue)
                return null;

            return _source.Repairs.FirstOrDefault(x => (!string.IsNullOrEmpty(Model.RepairName) && x.RepairName == Model.RepairName) || (Model.Id.HasValue && x.Id == Model.Id))?.GetViewModel;
        }
        
        public RepairViewModel? Insert(RepairBindingModel Model)
        {
            Model.Id = _source.Repairs.Count > 0 ? _source.Repairs.Max(x => x.Id) + 1 : 1;
            
            var NewRepair = Repair.Create(Model);
            
            if (NewRepair == null)
                return null;
 
            _source.Repairs.Add(NewRepair);
            _source.SaveRepairs();
            
            return NewRepair.GetViewModel;
        }
        
        public RepairViewModel? Update(RepairBindingModel Model)
        {
            var Repair = _source.Repairs.FirstOrDefault(x => x.Id == Model.Id);
            
            if (Repair == null)
                return null;

            Repair.Update(Model);
            _source.SaveRepairs();
            
            return Repair.GetViewModel;
        }
        
        public RepairViewModel? Delete(RepairBindingModel Model)
        {
            var Repair = _source.Repairs.FirstOrDefault(x => x.Id == Model.Id);

            if (Repair == null)
                return null;

            _source.Repairs.Remove(Repair);
            _source.SaveRepairs();
                
            return Repair.GetViewModel;
        }
    }
}
