﻿using AutoWorkshopContracts.BindingModels;
using AutoWorkshopContracts.ViewModels;
using AutoWorkshopDataModels.Models;
using System.Xml.Linq;

namespace AutoWorkshopFileImplement.Models
{
    public class Repair : IRepairModel
    {
        public int Id { get; private set; }
        
        public string RepairName { get; private set; } = string.Empty;
        
        public double Price { get; private set; }
        
        public Dictionary<int, int> Components { get; private set; } = new();
        
        private Dictionary<int, (IComponentModel, int)>? _RepairComponents = null;
        
        public Dictionary<int, (IComponentModel, int)> RepairComponents
        {
            get
            {
                if (_RepairComponents == null)
                {
                    var source = DataFileSingleton.GetInstance();
                    _RepairComponents = Components.ToDictionary(x => 
                        x.Key, y => ((source.Components.FirstOrDefault(z => z.Id == y.Key) as IComponentModel)!, y.Value));
                }
                return _RepairComponents;
            }
        }
        
        public static Repair? Create(RepairBindingModel Model)
        {
            if (Model is null)
                return null;

            return new Repair()
            {
                Id = Model.Id,
                RepairName = Model.RepairName,
                Price = Model.Price,
                Components = Model.RepairComponents.ToDictionary(x => x.Key, x => x.Value.Item2)
            };
        }
        public static Repair? Create(XElement Element)
        {
            if (Element is null)
                return null;

            return new Repair()
            {
                Id = Convert.ToInt32(Element.Attribute("Id")!.Value),
                RepairName = Element.Element("RepairName")!.Value,
                Price = Convert.ToDouble(Element.Element("Price")!.Value),
                Components = Element.Element("RepairComponents")!.Elements("RepairComponent").ToDictionary(x => 
                    Convert.ToInt32(x.Element("Key")?.Value), x => Convert.ToInt32(x.Element("Value")?.Value))
            };
        }
        
        public void Update(RepairBindingModel Model)
        {
            if (Model is null)
                return;

            RepairName = Model.RepairName;
            Price = Model.Price;
            Components = Model.RepairComponents.ToDictionary(x => x.Key, x => x.Value.Item2);
            _RepairComponents = null;
        }
        
        public RepairViewModel GetViewModel => new()
        {
            Id = Id,
            RepairName = RepairName,
            Price = Price,
            RepairComponents = RepairComponents
        };
        
        public XElement GetXElement => new(
            "Repair",
            new XAttribute("Id", Id),
            new XElement("RepairName", RepairName),
            new XElement("Price", Price.ToString()),
            new XElement("RepairComponents", Components.Select(x => 
                new XElement("RepairComponent", new XElement("Key", x.Key), new XElement("Value", x.Value))).ToArray())
        );
    }
}
