﻿using AutoWorkshopContracts.BindingModels;
using AutoWorkshopContracts.ViewModels;
using AutoWorkshopDataModels.Models;
using System.Xml.Linq;

namespace AutoWorkshopFileImplement.Models
{
    public class Shop : IShopModel
    {
        public int Id { get; private set; }
        
        public string ShopName { get; private set; } = string.Empty;
        
        public string Address { get; private set; } = string.Empty;
        
        public DateTime OpeningDate { get; private set; }
        
        public Dictionary<int, int> Repairs { get; private set; } = new();
        
        private Dictionary<int, (IRepairModel, int)>? _shopRepairs = null;

        public Dictionary<int, (IRepairModel, int)> ShopRepairs
        {
            get
            {
                if (_shopRepairs == null)
                {
                    var Source = DataFileSingleton.GetInstance();
                    _shopRepairs = Repairs.ToDictionary(x => x.Key, y => ((Source.Repairs.FirstOrDefault(z => z.Id == y.Key) as IRepairModel)!, y.Value));
                }
                
                return _shopRepairs;
            }
        }

        public int RepairsMaxCount { get; private set; }

        public static Shop? Create(ShopBindingModel? Model)
        {
            if (Model == null)
                return null;

            return new Shop()
            {
                Id = Model.Id,
                ShopName = Model.ShopName,
                Address = Model.Address,
                OpeningDate = Model.OpeningDate,
                Repairs = Model.ShopRepairs.ToDictionary(x => x.Key, x => x.Value.Item2),
                RepairsMaxCount = Model.RepairsMaxCount
            };
        }

        public static Shop? Create(XElement Element)
        {
            if (Element == null)
                return null;

            return new Shop()
            {
                Id = Convert.ToInt32(Element.Attribute("Id")!.Value),
                ShopName = Element.Element("ShopName")!.Value,
                Address = Element.Element("Address")!.Value,
                OpeningDate = Convert.ToDateTime(Element.Element("OpeningDate")!.Value),
                Repairs = Element.Element("ShopRepairs")!.Elements("ShopRepair")!.ToDictionary(x => Convert.ToInt32(x.Element("Key")?.Value),
                    x => Convert.ToInt32(x.Element("Value")?.Value)),
                RepairsMaxCount = Convert.ToInt32(Element.Element("RepairsMaxCount")!.Value)
            };
        }

        public void Update(ShopBindingModel? Model)
        {
            if (Model == null)
                return;

            ShopName = Model.ShopName;
            Address = Model.Address;
            OpeningDate = Model.OpeningDate;
            RepairsMaxCount = Model.RepairsMaxCount;
            Repairs = Model.ShopRepairs.ToDictionary(x => x.Key, x => x.Value.Item2);
            _shopRepairs = null;
        }

        public ShopViewModel GetViewModel => new()
        {
            Id = Id,
            ShopName = ShopName,
            Address = Address,
            OpeningDate = OpeningDate,
            ShopRepairs = ShopRepairs,
            RepairsMaxCount = RepairsMaxCount
        };

        public XElement GetXElement => new(
            "Shop",
            new XAttribute("Id", Id),
            new XElement("ShopName", ShopName),
            new XElement("Address", Address),
            new XElement("OpeningDate", OpeningDate.ToString()),
            new XElement("ShopRepairs", Repairs.Select(
                x => new XElement("ShopRepair", new XElement("Key", x.Key), new XElement("Value", x.Value))).ToArray()),
            new XElement("RepairsMaxCount", RepairsMaxCount.ToString())
        );

        public void RepairsUpdate()
        {
            _shopRepairs = null;
        }
    }
}
