﻿using AutoWorkshopContracts.BindingModels;
using AutoWorkshopContracts.SearchModels;
using AutoWorkshopContracts.StoragesContracts;
using AutoWorkshopContracts.ViewModels;
using AutoWorkshopListImplement.Models;

namespace AutoWorkshopListImplement.Implements
{
    public class RepairStorage : IRepairStorage
    {
        private readonly DataListSingleton _source;

        public RepairStorage()
        {
            _source = DataListSingleton.GetInstance();
        }

        public List<RepairViewModel> GetFullList()
        {
            var Result = new List<RepairViewModel>();

            foreach (var Repair in _source.Repairs)
            {
                Result.Add(Repair.GetViewModel);
            }

            return Result;
        }

        public List<RepairViewModel> GetFilteredList(RepairSearchModel Model)
        {
            var Result = new List<RepairViewModel>();

            if (string.IsNullOrEmpty(Model.RepairName))
                return Result;

            foreach (var Repair in _source.Repairs)
            {
                if (Repair.RepairName.Contains(Model.RepairName))
                {
                    Result.Add(Repair.GetViewModel);
                }
            }

            return Result;
        }

        public RepairViewModel? GetElement(RepairSearchModel Model)
        {
            if (string.IsNullOrEmpty(Model.RepairName) && !Model.Id.HasValue)
                return null;

            foreach (var Repair in _source.Repairs)
            {
                if ((!string.IsNullOrEmpty(Model.RepairName)
                    && Repair.RepairName == Model.RepairName)
                    || (Model.Id.HasValue && Repair.Id == Model.Id))
                {
                    return Repair.GetViewModel;
                }
            }

            return null;
        }

        public RepairViewModel? Insert(RepairBindingModel Model)
        {
            Model.Id = 1;
            foreach (var Repair in _source.Repairs)
            {
                if (Model.Id <= Repair.Id)
                {
                    Model.Id = Repair.Id + 1;
                }
            }

            var NewRepair = Repair.Create(Model);

            if (NewRepair == null)
                return null;

            _source.Repairs.Add(NewRepair);
            return NewRepair.GetViewModel;
        }

        public RepairViewModel? Update(RepairBindingModel Model)
        {
            foreach (var Repair in _source.Repairs)
            {
                if (Repair.Id == Model.Id)
                {
                    Repair.Update(Model);
                    return Repair.GetViewModel;
                }
            }

            return null;
        }

        public RepairViewModel? Delete(RepairBindingModel Model)
        {
            for (int i = 0; i < _source.Repairs.Count; ++i)
            {
                if (_source.Repairs[i].Id == Model.Id)
                {
                    var Element = _source.Repairs[i];
                    _source.Repairs.RemoveAt(i);

                    return Element.GetViewModel;
                }
            }

            return null;
        }
    }
}
