﻿using AutoWorkshopContracts.BindingModels;
using AutoWorkshopContracts.SearchModels;
using AutoWorkshopContracts.StoragesContracts;
using AutoWorkshopContracts.ViewModels;
using AutoWorkshopListImplement.Models;

namespace AutoWorkshopListImplement.Implements
{
    public class ShopStorage : IShopStorage
    {
        private readonly DataListSingleton _source;

        public ShopStorage()
        {
            _source = DataListSingleton.GetInstance();
        }

        public List<ShopViewModel> GetFullList()
        {
            var Result = new List<ShopViewModel>();
            
            foreach (var Shop in _source.Shops)
            {
                Result.Add(Shop.GetViewModel);
            }
            
            return Result;
        }

        public List<ShopViewModel> GetFilteredList(ShopSearchModel Model)
        {
            var Result = new List<ShopViewModel>();
            
            if (string.IsNullOrEmpty(Model.ShopName))
                return Result;

            foreach (var Shop in _source.Shops)
            {
                if (Shop.ShopName.Contains(Model.ShopName))
                    Result.Add(Shop.GetViewModel);
            }
            
            return Result;
        }

        public ShopViewModel? GetElement(ShopSearchModel Model)
        {
            if (string.IsNullOrEmpty(Model.ShopName) && !Model.Id.HasValue)
                return null;

            foreach (var Shop in _source.Shops)
            {
                if ((!string.IsNullOrEmpty(Model.ShopName) && Shop.ShopName == Model.ShopName) ||
                    (Model.Id.HasValue && Shop.Id == Model.Id))
                {
                    return Shop.GetViewModel;
                }
            }
            
            return null;
        }

        public ShopViewModel? Insert(ShopBindingModel Model)
        {
            Model.Id = 1;
            
            foreach (var Shop in _source.Shops)
            {
                if (Model.Id <= Shop.Id)
                    Model.Id = Shop.Id + 1;
            }
            
            var NewShop = Shop.Create(Model);
            
            if (NewShop is null)
                return null;
 
            _source.Shops.Add(NewShop);
            return NewShop.GetViewModel;
        }

        public ShopViewModel? Update(ShopBindingModel Model)
        {
            foreach (var Shop in _source.Shops)
            {
                if (Shop.Id == Model.Id)
                {
                    Shop.Update(Model);
                    return Shop.GetViewModel;
                }
            }
            
            return null;
        }

        public ShopViewModel? Delete(ShopBindingModel Model)
        {
            for (int i = 0; i < _source.Shops.Count; ++i)
            {
                if (_source.Shops[i].Id == Model.Id)
                {
                    var Shop = _source.Shops[i];
                    _source.Shops.RemoveAt(i);
                    
                    return Shop.GetViewModel;
                }
            }
            
            return null;
        }

        public bool Sell(SupplySearchModel Model)
        {
            throw new NotImplementedException();
        }

        public bool RestockingShops(SupplyBindingModel Model)
        {
            throw new NotImplementedException();
        }
    }
}
