﻿using AutoWorkshopContracts.BindingModels;
using AutoWorkshopContracts.ViewModels;
using AutoWorkshopDataModels.Enums;
using AutoWorkshopDataModels.Models;

namespace AutoWorkshopListImplement.Models
{
    public class Order : IOrderModel
    {
        public int Id { get; private set; }
        
        public int RepairId { get; private set; }

        public int ClientId { get; private set; }

        public int Count { get; private set; }
        
        public double Sum { get; private set; }
        
        public OrderStatus Status { get; private set; } = OrderStatus.Undefined;
        
        public DateTime DateCreate { get; private set; }
        
        public DateTime? DateImplement { get; private set; }

        public static Order? Create(OrderBindingModel? Model)
        {
            if (Model is null)
                return null;

            return new Order()
            {
                Id = Model.Id,
                RepairId = Model.RepairId,
                ClientId = Model.ClientId,
                Count = Model.Count,
                Sum = Model.Sum,
                Status = Model.Status,
                DateCreate = Model.DateCreate,
                DateImplement = Model.DateImplement
            };
        }

        public void Update(OrderBindingModel? Model)
        {
            if (Model == null)
                return;

            RepairId = Model.RepairId;
            Count = Model.Count;
            Sum = Model.Sum;
            Status = Model.Status;
            DateCreate = Model.DateCreate;
            DateImplement = Model.DateImplement;
        }

        public OrderViewModel GetViewModel => new()
        {
            Id = Id,
            RepairId = RepairId,
            ClientId = ClientId,
            Count = Count,
            Sum = Sum,
            Status = Status,
            DateCreate = DateCreate,
            DateImplement = DateImplement
        };
    }
}
