﻿using AutoWorkshopContracts.BindingModels;
using AutoWorkshopContracts.ViewModels;
using AutoWorkshopDataModels.Models;

namespace AutoWorkshopListImplement.Models
{
    public class Shop : IShopModel
    {
        public int Id { get; private set; }
        
        public string ShopName { get; private set; } = string.Empty;
        
        public string Address { get; private set; } = string.Empty;
        
        public DateTime OpeningDate { get; private set; }
        
        public Dictionary<int, (IRepairModel, int)> ShopRepairs { get; private set; } = new();

        public int RepairsMaxCount { get; private set; }

        public static Shop? Create(ShopBindingModel? Model)
        {
            if (Model is null)
                return null;

            return new Shop()
            {
                Id = Model.Id,
                ShopName = Model.ShopName,
                Address = Model.Address,
                OpeningDate = Model.OpeningDate,
                RepairsMaxCount = Model.RepairsMaxCount,
            };
        }

        public void Update(ShopBindingModel? Model)
        {
            if (Model is null)
                return;

            ShopName = Model.ShopName;
            Address = Model.Address;
            OpeningDate = Model.OpeningDate;
            RepairsMaxCount = Model.RepairsMaxCount;
        }

        public ShopViewModel GetViewModel => new()
        {
            Id = Id,
            ShopName = ShopName,
            Address = Address,
            OpeningDate = OpeningDate,
            ShopRepairs = ShopRepairs,
            RepairsMaxCount = RepairsMaxCount,
        };
    }
}
