﻿using AutoWorkshopContracts.BindingModels;
using AutoWorkshopContracts.BusinessLogicContracts;
using AutoWorkshopContracts.SearchModels;
using AutoWorkshopContracts.ViewModels;
using Microsoft.AspNetCore.Mvc;

namespace AutoWorkshopRestApi.Controllers
{
    [Route("api/[controller]/[action]")]
    [ApiController]
    public class MainController : Controller
    {
        private readonly ILogger _logger;
        private readonly IOrderLogic _order;
        private readonly IRepairLogic _repair;
        
        public MainController(ILogger<MainController> Logger, IOrderLogic Order, IRepairLogic Repair)
        {
            _logger = Logger;
            _order = Order;
            _repair = Repair;
        }
        
        [HttpGet]
        public List<RepairViewModel>? GetRepairList()
        {
            try
            {
                return _repair.ReadList(null);
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка получения списка продуктов");
                throw;
            }
        }
        
        [HttpGet]
        public RepairViewModel? GetRepair(int RepairId)
        {
            try
            {
                return _repair.ReadElement(new RepairSearchModel
                {
                    Id = RepairId
                });
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка получения продукта по id={Id}", RepairId);
                throw;
            }
        }
        
        [HttpGet]
        public List<OrderViewModel>? GetOrders(int ClientId)
        {
            try
            {
                return _order.ReadList(new OrderSearchModel
                {
                    ClientId = ClientId
                });
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка получения списка заказов клиента id ={ Id} ", ClientId);
                throw;
            }
        }
        
        [HttpPost]
        public void CreateOrder(OrderBindingModel Model)
        {
            try
            {
                _order.CreateOrder(Model);
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка создания заказа");
                throw;
            }
        }
    }
}
