﻿using AutoWorkshopContracts.BindingModels;
using AutoWorkshopContracts.BusinessLogicsContracts;
using AutoWorkshopContracts.SearchModels;
using AutoWorkshopContracts.ViewModels;
using Microsoft.AspNetCore.Mvc;

namespace AutoWorkshopRestApi.Controllers
{
    [Route("api/[controller]/[action]")]
    [ApiController]
    public class ShopController : Controller
    {
        private readonly ILogger _logger;
        private readonly IShopLogic _shopLogic;

        public ShopController(ILogger<ShopController> Logger, IShopLogic ShopLogic)
        {
            _logger = Logger;
            _shopLogic = ShopLogic;
        }

        [HttpGet]
        public List<ShopViewModel>? GetShopList()
        {
            try
            {
                return _shopLogic.ReadList(null);
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка получения списка магазинов");
                throw;
            }
        }

        [HttpGet]
        public ShopRepairsViewModel? GetShop(int ShopId)
        {
            try
            {
                var Shop = _shopLogic.ReadElement(new ShopSearchModel { Id = ShopId });

                if (Shop == null)
                    return null;

                return new ShopRepairsViewModel
                {
                    Shop = Shop,
                    ShopRepairs = Shop.ShopRepairs.ToDictionary(x => x.Key, x => new RepairCount
                    {
                        Repair = new RepairViewModel()
                        {
                            Id = x.Value.Item1.Id,
                            RepairName = x.Value.Item1.RepairName,
                            RepairComponents = x.Value.Item1.RepairComponents,
                            Price = x.Value.Item1.Price,
                        },
                        Count = x.Value.Item2
                    })
                };
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка получения магазина");
                throw;
            }
        }

        [HttpPost]
        public void CreateShop(ShopBindingModel Model)
        {
            try
            {
                _shopLogic.Create(Model);
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка создания магазина");
                throw;
            }
        }

        [HttpPost]
        public void UpdateShop(ShopBindingModel Model)
        {
            try
            {
                _shopLogic.Update(Model);
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка обновления магазина");
                throw;
            }
        }

        [HttpPost]
        public void DeleteShop(ShopBindingModel Model)
        {
            try
            {
                _shopLogic.Delete(Model);
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка удаления магазина");
                throw;
            }
        }

        [HttpPost]
        public void MakeSupply(SupplyBindingModel Model)
        {
            try
            {
                _shopLogic.MakeSupply(Model);
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка создания поставки в магазин");
                throw;
            }
        }
    }
}
