using AutoWorkshopBusinessLogic.BusinessLogics;
using AutoWorkshopContracts.BusinessLogicContracts;
using AutoWorkshopContracts.BusinessLogicsContracts;
using AutoWorkshopContracts.StoragesContracts;
using AutoWorkshopDatabaseImplement.Implements;
using Microsoft.OpenApi.Models;

var Builder = WebApplication.CreateBuilder(args);
Builder.Logging.SetMinimumLevel(LogLevel.Trace);
Builder.Logging.AddLog4Net("log4net.config");

// Add services to the container.
Builder.Services.AddTransient<IClientStorage, ClientStorage>();
Builder.Services.AddTransient<IOrderStorage, OrderStorage>();
Builder.Services.AddTransient<IRepairStorage, RepairStorage>();
Builder.Services.AddTransient<IShopStorage, ShopStorage>();

Builder.Services.AddTransient<IOrderLogic, OrderLogic>();
Builder.Services.AddTransient<IClientLogic, ClientLogic>();
Builder.Services.AddTransient<IRepairLogic, RepairLogic>();
Builder.Services.AddTransient<IShopLogic, ShopLogic>();

Builder.Services.AddControllers();

Builder.Services.AddEndpointsApiExplorer();
Builder.Services.AddSwaggerGen(c =>
{
    c.SwaggerDoc("v1", new OpenApiInfo
    {
        Title = "AutoWorkshopRestApi",
        Version = "v1"
    });
});

var App = Builder.Build();

if (App.Environment.IsDevelopment())
{
    App.UseSwagger();
    App.UseSwaggerUI(c => c.SwaggerEndpoint("/swagger/v1/swagger.json",
   "AutoWorkshopRestApi v1"));
}

App.UseHttpsRedirection();
App.UseAuthorization();
App.MapControllers();

App.Run();
