﻿using AutoWorkshopContracts.BindingModels;
using AutoWorkshopContracts.ViewModels;
using AutoWorkshopShopApp.Models;
using Microsoft.AspNetCore.Mvc;
using System.Diagnostics;

namespace AutoWorkshopShopApp.Controllers
{
    public class HomeController : Controller
    {
        private readonly ILogger<HomeController> _logger;

        public HomeController(ILogger<HomeController> Logger)
        {
            _logger = Logger;
        }

        public IActionResult Index()
        {
            if (!ApiClient.IsAuthenticated)
            {
                return Redirect("~/Home/Enter");
            }
            return View(ApiClient.GetRequest<List<ShopViewModel>>("api/shop/getshoplist"));
        }

        [HttpGet]
        public IActionResult Enter()
        {
            return View();
        }

        [HttpPost]
        public void Enter(string Password)
        {
			if (string.IsNullOrEmpty(Password))
			{
				throw new Exception("Введите пароль");
			}
			if (!ApiClient.TryAuthenticate(Password))
			{
				throw new Exception("Неверный пароль");
			}
			
            Response.Redirect("Index");
		}

        [HttpGet]
        public IActionResult Create()
        {
            if (!ApiClient.IsAuthenticated)
            {
                return Redirect("~/Home/Enter");
            }
            return View("Shop");
        }

        [HttpPost]
        public void Create(int Id, string ShopName, string Address, DateTime OpeningDate, int MaxCount)
        {
			if (!ApiClient.IsAuthenticated)
			{
				throw new Exception("Вход только авторизованным");
			}

			if (string.IsNullOrEmpty(ShopName) || string.IsNullOrEmpty(Address))
            {
                throw new Exception("Название или адрес не может быть пустым");
            }
			if (MaxCount <= 0)
			{
				throw new Exception("Вместимость магазина должна быть больше нуля");
			}

			ApiClient.PostRequest("api/shop/createshop", new ShopBindingModel
            {
                Id = Id,
                ShopName = ShopName,
                Address = Address,
                OpeningDate = OpeningDate,
                RepairsMaxCount = MaxCount
            });
            Response.Redirect("Index");
        }

        [HttpGet]
        public IActionResult Update(int Id)
        {
			if (!ApiClient.IsAuthenticated)
			{
				return Redirect("~/Home/Enter");
			}
			return View("Shop", ApiClient.GetRequest<ShopRepairsViewModel>($"api/shop/getshop?shopId={Id}"));
        }

        [HttpPost]
        public void Update(int Id, string ShopName, string Address, DateTime OpeningDate, int MaxCount)
        {
			if (!ApiClient.IsAuthenticated)
			{
				throw new Exception("Вход только авторизованным");
			}

			if (string.IsNullOrEmpty(ShopName) || string.IsNullOrEmpty(Address))
            {
                throw new Exception("Название или адрес не может быть пустым");
            }
			if (MaxCount <= 0)
			{
				throw new Exception("Вместимость магазина должна быть больше нуля");
			}
			
            ApiClient.PostRequest("api/shop/updateshop", new ShopBindingModel
            {
                Id = Id,
                ShopName = ShopName,
                Address = Address,
                OpeningDate = OpeningDate,
                RepairsMaxCount = MaxCount
            });
            Response.Redirect("Index");
        }

        [HttpPost]
        public void Delete(int Id)
        {
			if (!ApiClient.IsAuthenticated)
			{
				throw new Exception("Вход только авторизованным");
			}

			ApiClient.PostRequest("api/shop/deleteshop", new ShopBindingModel
			{
				Id = Id,
			});
			Response.Redirect("Index");
        }

        [HttpGet]
        public IActionResult Supply()
        {
			if (!ApiClient.IsAuthenticated)
			{
				return Redirect("~/Home/Enter");
			}

			ViewBag.Shops = ApiClient.GetRequest<List<ShopViewModel>>("api/shop/getshoplist");
            ViewBag.Repairs = ApiClient.GetRequest<List<RepairViewModel>>("api/main/getrepairlist");
            return View();
        }

        [HttpPost]
        public void Supply(int Shop, int Repair, int Count)
        {
			if (!ApiClient.IsAuthenticated)
			{
				throw new Exception("Вход только авторизованным");
			}

			ApiClient.PostRequest("api/shop/makesupply", new SupplyBindingModel
            {
                ShopId = Shop,
                RepairId = Repair,
                Count = Count
            });
            Response.Redirect("Index");
        }

        [ResponseCache(Duration = 0, Location = ResponseCacheLocation.None, NoStore = true)]
        public IActionResult Error()
        {
            return View(new ErrorViewModel { RequestId = Activity.Current?.Id ?? HttpContext.TraceIdentifier });
        }
    }
}
