﻿using AutoWorkshopContracts.BindingModels;
using AutoWorkshopContracts.BusinessLogicContracts;
using AutoWorkshopContracts.SearchModels;
using Microsoft.Extensions.Logging;

namespace AutoWorkshopView.Forms
{
    public partial class FormCreateOrder : Form
    {
        private readonly ILogger _logger;
        private readonly IRepairLogic _repairLogic;
        private readonly IOrderLogic _orderLogic;
        private readonly IClientLogic _clientLogic;

        public FormCreateOrder(ILogger<FormCreateOrder> Logger, IRepairLogic RepairLogic, IOrderLogic OrderLogic, IClientLogic ClientLogic)
        {
            InitializeComponent();

            _logger = Logger;
            _repairLogic = RepairLogic;
            _orderLogic = OrderLogic;
            _clientLogic = ClientLogic;
        }

        private void FormCreateOrder_Load(object sender, EventArgs e)
        {
            _logger.LogInformation("Загрузка ремонтов для заказа");
            
            try
            {
                var List = _repairLogic.ReadList(null);
                if (List != null)
                {
                    RepairComboBox.DisplayMember = "RepairName";
                    RepairComboBox.ValueMember = "Id";
                    RepairComboBox.DataSource = List;
                    RepairComboBox.SelectedItem = null;
                }
                
                _logger.LogInformation("Ремонты загружены");
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка загрузки ремонтов");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }

			_logger.LogInformation("Загрузка клиентов для заказа");
			try
			{
				var list = _clientLogic.ReadList(null);
				if (list != null)
				{
					comboBoxClient.DisplayMember = "ClientFIO";
					comboBoxClient.ValueMember = "Id";
					comboBoxClient.DataSource = list;
					comboBoxClient.SelectedItem = null;
				}
				_logger.LogInformation("Клиенты загружены");
			}
			catch (Exception ex)
			{
				_logger.LogError(ex, "Ошибка загрузки клиентов");
				MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK,
			   MessageBoxIcon.Error);
			}
		}

        private void CalcSum()
        {
            if (RepairComboBox.SelectedValue != null && !string.IsNullOrEmpty(CountTextBox.Text))
            {
                try
                {
                    int id = Convert.ToInt32(RepairComboBox.SelectedValue);
                    var Repair = _repairLogic.ReadElement(new RepairSearchModel
                    {
                        Id = id
                    });
                    int Count = Convert.ToInt32(CountTextBox.Text);

                    SumTextBox.Text = Math.Round(Count * (Repair?.Price ?? 0), 2).ToString();
                    
                    _logger.LogInformation("Расчет суммы заказа");
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Ошибка расчета суммы заказа");
                    MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }

        private void CountTextBox_TextChanged(object sender, EventArgs e)
        {
            CalcSum();
        }

        private void SaveButton_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(CountTextBox.Text))
            {
                MessageBox.Show("Заполните поле Количество", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            if (RepairComboBox.SelectedValue == null)
            {
                MessageBox.Show("Выберите ремонт", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            _logger.LogInformation("Создание заказа");

            try
            {
                var OperationResult = _orderLogic.CreateOrder(new OrderBindingModel
                {
                    RepairId = Convert.ToInt32(RepairComboBox.SelectedValue),
                    Count = Convert.ToInt32(CountTextBox.Text),
                    Sum = Convert.ToDouble(SumTextBox.Text)
                });
                if (!OperationResult)
                {
                    throw new Exception("Ошибка при создании заказа. Дополнительная информация в логах.");
                }

                MessageBox.Show("Сохранение прошло успешно", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
                DialogResult = DialogResult.OK;
                Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка создания заказа");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void CancelButton_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }

        private void RepairComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            CalcSum();
        }
    }
}
