﻿using AutoWorkshopContracts.BindingModels;
using AutoWorkshopContracts.BusinessLogicContracts;
using Microsoft.Extensions.Logging;

namespace AutoWorkshopView.Forms
{
    public partial class FormRepairs : Form
    {
        private readonly ILogger _logger;
        private readonly IRepairLogic _logic;

        public FormRepairs(ILogger<FormRepairs> Logger, IRepairLogic Logic)
        {
            InitializeComponent();

            _logger = Logger;
            _logic = Logic;
        }

        private void FormRepairs_Load(object sender, EventArgs e)
        {
            LoadData();
        }

        private void LoadData()
        {
            try
            {
                var List = _logic.ReadList(null);
                
                if (List != null)
                {
                    DataGridView.DataSource = List;
                    DataGridView.Columns["Id"].Visible = false;
                    DataGridView.Columns["RepairName"].AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;
                    DataGridView.Columns["RepairComponents"].Visible = false;
                }
                
                _logger.LogInformation("Загрузка ремонта");
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка загрузки ремонта");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void AddButton_Click(object sender, EventArgs e)
        {
            var Service = Program.ServiceProvider?.GetService(typeof(FormRepair));
            
            if (Service is FormRepair Form)
            {
                if (Form.ShowDialog() == DialogResult.OK)
                {
                    LoadData();
                }
            }
        }

        private void UpdateButton_Click(object sender, EventArgs e)
        {
            if (DataGridView.SelectedRows.Count == 1)
            {
                var Service = Program.ServiceProvider?.GetService(typeof(FormRepair));
                
                if (Service is FormRepair Form)
                {
                    var Temp = Convert.ToInt32(DataGridView.SelectedRows[0].Cells["Id"].Value);
                    Form.Id = Convert.ToInt32(DataGridView.SelectedRows[0].Cells["Id"].Value);
                    
                    if (Form.ShowDialog() == DialogResult.OK)
                    {
                        LoadData();
                    }
                }
            }
        }

        private void DeleteButton_Click(object sender, EventArgs e)
        {
            if (DataGridView.SelectedRows.Count == 1)
            {
                if (MessageBox.Show("Удалить запись?", "Вопрос", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                {
                    int id = Convert.ToInt32(DataGridView.SelectedRows[0].Cells["Id"].Value);
                    _logger.LogInformation("Удаление ремонта");
                    try
                    {
                        if (!_logic.Delete(new RepairBindingModel
                        {
                            Id = id
                        }))
                        {
                            throw new Exception("Ошибка при удалении. Дополнительная информация в логах.");
                        }
                        
                        LoadData();
                    }
                    catch (Exception ex)
                    {
                        _logger.LogError(ex, "Ошибка удаления ремонта");
                        MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
            }
        }

        private void RefreshButton_Click(object sender, EventArgs e)
        {
            LoadData();
        }
    }
}
