﻿using AutoWorkshopContracts.BindingModels;
using AutoWorkshopContracts.BusinessLogicContracts;
using Microsoft.Extensions.Logging;
using Microsoft.Reporting.WinForms;

namespace AutoWorkshopView.Forms
{
    public partial class FormReportGroupedOrders : Form
    {
        private readonly ReportViewer _reportViewer;
        private readonly ILogger _logger;
        private readonly IReportLogic _logic;

        public FormReportGroupedOrders(ILogger<FormReportGroupedOrders> Logger, IReportLogic Logic)
        {
            InitializeComponent();
            _logger = Logger;
            _logic = Logic;
            
            _reportViewer = new ReportViewer
            {
                Dock = DockStyle.Fill
            };
            _reportViewer.LocalReport.LoadReportDefinition(new FileStream("ReportGroupedOrders.rdlc", FileMode.Open));
            
            Controls.Clear();
            Controls.Add(_reportViewer);
            Controls.Add(Panel);
        }

        private void ButtonMake_Click(object sender, EventArgs e)
        {
            try
            {
                var DataSource = _logic.GetGroupedOrders();
                var Source = new ReportDataSource("DataSetGroupedOrders", DataSource);
                _reportViewer.LocalReport.DataSources.Clear();
                _reportViewer.LocalReport.DataSources.Add(Source);

                _reportViewer.RefreshReport();
                _logger.LogInformation("Загрузка списка заказов, объединенных по датам");
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка загрузки списка заказов, объединенных по датам");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonToPdf_Click(object sender, EventArgs e)
        {
            using var Dialog = new SaveFileDialog { Filter = "pdf|*.pdf" };
            
            if (Dialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    System.Text.Encoding.RegisterProvider(System.Text.CodePagesEncodingProvider.Instance);
                    _logic.SaveGroupedOrdersToPdfFile(new ReportBindingModel
                    {
                        FileName = Dialog.FileName,
                    });
                    
                    _logger.LogInformation("Сохранение списка заказов, объединенных по датам");
                    MessageBox.Show("Выполнено", "Успех", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Ошибка сохранения списка заказов, объединенных по датам");
                    MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }
    }
}
