﻿using AutoWorkshopContracts.BindingModels;
using AutoWorkshopContracts.BusinessLogicContracts;
using Microsoft.Extensions.Logging;

namespace AutoWorkshopView.Forms
{
    public partial class FormReportRepairComponents : Form
    {
        private readonly ILogger _logger;
        private readonly IReportLogic _logic;

        public FormReportRepairComponents(ILogger<FormReportRepairComponents> Logger, IReportLogic Logic)
        {
            InitializeComponent();
            
            _logger = Logger;
            _logic = Logic;
        }

        private void FormReportRepairComponents_Load(object sender, EventArgs e)
        {
            try
            {
                var Dict = _logic.GetRepairComponents();
                
                if (Dict != null)
                {
                    DataGridView.Rows.Clear();
                    
                    foreach (var Elem in Dict)
                    {
                        DataGridView.Rows.Add(new object[] { Elem.RepairName, "", "" });
                        
                        foreach (var ListElem in Elem.Components)
                            DataGridView.Rows.Add(new object[] { "", ListElem.Item1, ListElem.Item2 });

                        DataGridView.Rows.Add(new object[] { "Итого", "", Elem.TotalCount });
                        DataGridView.Rows.Add(Array.Empty<object>());
                    }
                }
                
                _logger.LogInformation("Загрузка списка ремонтов по компонентам");
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка загрузки списка ремонтов по компонентам");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonSaveToExcel_Click(object sender, EventArgs e)
        {
            using var Dialog = new SaveFileDialog { Filter = "xlsx|*.xlsx" };
            
            if (Dialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    _logic.SaveRepairComponentToExcelFile(new ReportBindingModel
                    {
                        FileName = Dialog.FileName
                    });
                    
                    _logger.LogInformation("Сохранение списка ремонтов по компонентам");
                    MessageBox.Show("Выполнено", "Успех", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Ошибка сохранения списка ремонтов по компонентам");
                    MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }
    }
}
