﻿using AutoWorkshopContracts.BindingModels;
using AutoWorkshopContracts.BusinessLogicContracts;
using AutoWorkshopContracts.BusinessLogicsContracts;
using AutoWorkshopContracts.ViewModels;
using Microsoft.Extensions.Logging;

namespace AutoWorkshopView.Forms.Shop
{
    public partial class FormCreateSupply : Form
    {
        private readonly ILogger _logger;
        private readonly IRepairLogic _repairLogic;
        private readonly IShopLogic _shopLogic;
        
        private List<ShopViewModel> _shopList = new List<ShopViewModel>();
        private List<RepairViewModel> _RepairList = new List<RepairViewModel>();

        public FormCreateSupply(ILogger<FormCreateSupply> Logger, IRepairLogic RepairLogic, IShopLogic ShopLogic)
        {
            InitializeComponent();
            
            _logger = Logger;
            _repairLogic = RepairLogic;
            _shopLogic = ShopLogic;
        }

        private void FormCreateSupply_Load(object sender, EventArgs e)
        {
            _shopList = _shopLogic.ReadList(null);
            _RepairList = _repairLogic.ReadList(null);
            
            if (_shopList != null)
            {
                ShopComboBox.DisplayMember = "ShopName";
                ShopComboBox.ValueMember = "Id";
                ShopComboBox.DataSource = _shopList;
                ShopComboBox.SelectedItem = null;
                _logger.LogInformation("Загрузка магазинов для поставок");
            }
            
            if (_RepairList != null)
            {
                RepairComboBox.DisplayMember = "RepairName";
                RepairComboBox.ValueMember = "Id";
                RepairComboBox.DataSource = _RepairList;
                RepairComboBox.SelectedItem = null;
                _logger.LogInformation("Загрузка ремонтов для поставок");
            }
        }

        private void SaveButton_Click(object sender, EventArgs e)
        {
            if (ShopComboBox.SelectedValue == null)
            {
                MessageBox.Show("Выберите магазин", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            
            if (RepairComboBox.SelectedValue == null)
            {
                MessageBox.Show("Выберите изделие", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            
            _logger.LogInformation("Создание поставки");
            
            try
            {
                var OperationResult = _shopLogic.MakeSupply(new SupplyBindingModel
                {
                    ShopId = Convert.ToInt32(ShopComboBox.SelectedValue),
                    RepairId = Convert.ToInt32(RepairComboBox.SelectedValue),
                    Count = Convert.ToInt32(CountTextbox.Text)
                });
                
                if (!OperationResult)
                {
                    throw new Exception("Ошибка при создании поставки. Дополнительная информация в логах.");
                }
                
                MessageBox.Show("Сохранение прошло успешно", "Сообщение", MessageBoxButtons.OK, MessageBoxIcon.Information);
                
                DialogResult = DialogResult.OK;
                Close();
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка создания поставки");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void CancelButton_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }
    }
}
