﻿using AutoWorkshopContracts.BusinessLogicContracts;
using AutoWorkshopContracts.BusinessLogicsContracts;
using AutoWorkshopContracts.SearchModels;
using AutoWorkshopContracts.ViewModels;
using Microsoft.Extensions.Logging;

namespace AutoWorkshopView.Forms.Shop
{
    public partial class FormSellRepair : Form
    {
        private readonly ILogger _logger;

        private readonly IRepairLogic _repairLogic;
        private readonly IShopLogic _shopLogic;

        private List<RepairViewModel> _repairList = new List<RepairViewModel>();

        public FormSellRepair(ILogger<FormSellRepair> Logger, IRepairLogic RepairLogic, IShopLogic ShopLogic)
        {
            InitializeComponent();

            _logger = Logger;
            _repairLogic = RepairLogic;
            _shopLogic = ShopLogic;
        }

        private void FormSellRepair_Load(object sender, EventArgs e)
        {
            _repairList = _repairLogic.ReadList(null);

            if (_repairList != null)
            {
                RepairComboBox.DisplayMember = "RepairName";
                RepairComboBox.ValueMember = "Id";
                RepairComboBox.DataSource = _repairList;
                RepairComboBox.SelectedItem = null;

                _logger.LogInformation("Загрузка ремонтов для продажи");
            }
        }

        private void SellButton_Click(object sender, EventArgs e)
        {
            if (RepairComboBox.SelectedValue == null)
            {
                MessageBox.Show("Выберите ремонт", "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }

            _logger.LogInformation("Создание покупки");

            try
            {
                bool Result = _shopLogic.MakeSell(new SupplySearchModel
                {
                    RepairId = Convert.ToInt32(RepairComboBox.SelectedValue),
                    Count = Convert.ToInt32(CountTextBox.Text)
                });

                if (Result)
                {
                    _logger.LogInformation("Проверка пройдена, продажа проведена");
                    MessageBox.Show("Продажа проведена", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Information);

                    DialogResult = DialogResult.OK;
                    Close();
                }
                else
                {
                    _logger.LogInformation("Проверка не пройдена");
                    MessageBox.Show("Продажа не может быть создана.", "Результат", MessageBoxButtons.OK, MessageBoxIcon.Warning);
                }
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка создания покупки");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonCancel_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.Cancel;
            Close();
        }
    }
}
