﻿using AutoWorkshopContracts.BindingModels;
using AutoWorkshopContracts.BusinessLogicsContracts;
using Microsoft.Extensions.Logging;

namespace AutoWorkshopView.Forms.Shop
{
    public partial class FormShops : Form
    {
        private readonly ILogger _logger;
        private readonly IShopLogic _logic;

        public FormShops(ILogger<FormShops> Logger, IShopLogic Logic)
        {
            InitializeComponent();
            
            _logger = Logger;
            _logic = Logic;
        }

        private void FormShops_Load(object sender, EventArgs e)
        {
            LoadData();
        }

        private void LoadData()
        {
            try
            {
                var List = _logic.ReadList(null);
                
                if (List != null)
                {
                    ViewDataGrid.DataSource = List;
                    ViewDataGrid.Columns["Id"].Visible = false;
                    ViewDataGrid.Columns["ShopRepairs"].Visible = false;
                    ViewDataGrid.Columns["ShopName"].AutoSizeMode = DataGridViewAutoSizeColumnMode.Fill;
                }
                
                _logger.LogInformation("Загрузка магазинов");
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка загрузки магазинов");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void AddButton_Click(object sender, EventArgs e)
        {
            var Service = Program.ServiceProvider?.GetService(typeof(FormShop));
            
            if (Service is FormShop Form)
            {
                if (Form.ShowDialog() == DialogResult.OK)
                {
                    LoadData();
                }
            }
        }

        private void UpdateButton_Click(object sender, EventArgs e)
        {
            if (ViewDataGrid.SelectedRows.Count == 1)
            {
                var Service = Program.ServiceProvider?.GetService(typeof(FormShop));                
                if (Service is FormShop Form)
                {
                    Form.Id = Convert.ToInt32(ViewDataGrid.SelectedRows[0].Cells["Id"].Value);
                    
                    if (Form.ShowDialog() == DialogResult.OK)
                    {
                        LoadData();
                    }
                }
            }
        }

        private void DeleteButton_Click(object sender, EventArgs e)
        {
            if (ViewDataGrid.SelectedRows.Count == 1)
            {
                if (MessageBox.Show("Удалить запись?", "Вопрос", MessageBoxButtons.YesNo, MessageBoxIcon.Question) == DialogResult.Yes)
                {
                    int Id = Convert.ToInt32(ViewDataGrid.SelectedRows[0].Cells["Id"].Value);
                    _logger.LogInformation("Удаление магазина");
                    
                    try
                    {
                        if (!_logic.Delete(new ShopBindingModel
                            {
                                Id = Id
                            }))
                        {
                            throw new Exception("Ошибка при удалении. Дополнительная информация в логах.");
                        }
                        
                        LoadData();
                    }
                    catch (Exception ex)
                    {
                        _logger.LogError(ex, "Ошибка удаления магазина");
                        MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    }
                }
            }
        }

        private void RefreshButton_Click(object sender, EventArgs e)
        {
            LoadData();
        }
    }
}
