using AutoWorkshopBusinessLogic.BusinessLogics;
using AutoWorkshopBusinessLogic.OfficePackage.Implements;
using AutoWorkshopBusinessLogic.OfficePackage;
using AutoWorkshopContracts.BusinessLogicContracts;
using AutoWorkshopContracts.BusinessLogicsContracts;
using AutoWorkshopContracts.StoragesContracts;
using AutoWorkshopDatabaseImplement.Implements;
using AutoWorkshopView.Forms;
using AutoWorkshopView.Forms.Shop;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Logging;
using NLog.Extensions.Logging;

namespace AutoWorkshopView
{
    internal static class Program
    {
        private static ServiceProvider? _serviceProvider;
        public static ServiceProvider? ServiceProvider => _serviceProvider;

        [STAThread]
        static void Main()
        {
            ApplicationConfiguration.Initialize();

            var Services = new ServiceCollection();
            ConfigureServices(Services);
            
            _serviceProvider = Services.BuildServiceProvider();            
            Application.Run(_serviceProvider.GetRequiredService<MainForm>());
        }

        private static void ConfigureServices(ServiceCollection Services)
        {
            Services.AddLogging(option =>
            {
                option.SetMinimumLevel(LogLevel.Information);
                option.AddNLog("nlog.config");
            });

            Services.AddTransient<IComponentStorage, ComponentStorage>();
            Services.AddTransient<IOrderStorage, OrderStorage>();
            Services.AddTransient<IRepairStorage, RepairStorage>();
            Services.AddTransient<IShopStorage, ShopStorage>();
            Services.AddTransient<IClientStorage, ClientStorage>();

            Services.AddTransient<IComponentLogic, ComponentLogic>();
            Services.AddTransient<IOrderLogic, OrderLogic>();
            Services.AddTransient<IRepairLogic, RepairLogic>();
            Services.AddTransient<IReportLogic, ReportLogic>();
            Services.AddTransient<IShopLogic, ShopLogic>();
			Services.AddTransient<IClientLogic, ClientLogic>();			

			Services.AddTransient<AbstractSaveToWord, SaveToWord>();
            Services.AddTransient<AbstractSaveToExcel, SaveToExcel>();
            Services.AddTransient<AbstractSaveToPdf, SaveToPdf>();

            Services.AddTransient<MainForm>();
            Services.AddTransient<FormComponent>();
            Services.AddTransient<FormComponents>();
            Services.AddTransient<FormCreateOrder>();
            Services.AddTransient<FormRepair>();
            Services.AddTransient<FormRepairComponent>();
            Services.AddTransient<FormRepairs>();
            Services.AddTransient<FormShop>();
            Services.AddTransient<FormShops>();
            Services.AddTransient<FormCreateSupply>();
            Services.AddTransient<FormSellRepair>();
            Services.AddTransient<FormReportRepairComponents>();
            Services.AddTransient<FormReportOrders>();
            Services.AddTransient<FormReportShop>();
            Services.AddTransient<FormReportGroupedOrders>();
			Services.AddTransient<FormClients>();
		}
    }
}
